# -*- encoding: utf-8 -*-
from django.core.exceptions import ImproperlyConfigured
from django.db import models
from django.template.loader import get_template
from django.utils.translation import ugettext_lazy as _

from .behaviors import Orderable, Titled
from .validators import TemplateValidator


class ReportElement(Titled, Orderable):
    parent = models.ForeignKey('flexible_reports.Report')
    datasource = models.ForeignKey('flexible_reports.Datasource')
    table = models.ForeignKey('flexible_reports.Table')
    slug = models.SlugField(max_length=200)

    class Meta:
        unique_together = [
            ('parent', 'position'),
            ('parent', 'slug')
        ]
        ordering = ('position',)


def _get_template(name):
    t = get_template(name)
    return t.template.source


def get_reports_template():
    return _get_template("flexible_reports/report.html")


class Report(Titled):
    """Report is a collection of multiple Datasources' data
    rendered in Tables. """

    slug = models.SlugField()

    template = models.TextField(
        verbose_name=_("Template"),
        default=get_reports_template,
        validators=[TemplateValidator, ]
    )

    _base_queryset = None
    _context = None

    @property
    def base_queryset(self):
        if self._base_queryset is None:
            raise ImproperlyConfigured("Please set base queryset for this "
                                       "report using Report.set_base_queryset")
        return self._base_queryset

    @property
    def context(self):
        return self._context

    class Meta:
        verbose_name_plural = _("Reports")
        verbose_name = _("Report")

    def set_base_queryset(self, queryset):
        self._base_queryset = queryset

    def set_context(self, context):
        self._context = context



# -*- encoding: utf-8 -*-
from django import template
from django.template.base import Template

from ..adapters import django_tables2

register = template.Library()


@register.simple_tag(takes_context=True)
def flexible(context, report):
    return django_tables2.as_html(report, parent_context=context)


@register.simple_tag(takes_context=True)
def render(context, value):
    return Template(value).render(context)



# -*- encoding: utf-8 -*-
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils.translation import ugettext_lazy as _


class WithBaseModel(models.Model):
    base_model = models.ForeignKey(
        ContentType,
        verbose_name=_("Base model"),
    )

    class Meta:
        abstract = True


class Labelled(models.Model):
    label = models.TextField(
        verbose_name=_("Label")
    )

    def __str__(self):
        return self.label

    class Meta:
        abstract = True
        ordering = ('label',)


class Titled(models.Model):
    title = models.TextField(
        verbose_name=_("Title")
    )

    def __str__(self):
        return self.title

    class Meta:
        abstract = True


class Orderable(models.Model):
    """Can be placed in an order. Has a position. This field name is the
    default for Grappelli. """
    position = models.PositiveIntegerField(
        default=0,
        verbose_name=_("Position"))

    class Meta:
        abstract = True
        ordering = ('position',)



# -*- encoding: utf-8 -*-
from django.db import models
from django.utils.translation import ugettext_lazy as _
from django_dsl import compiler
from django_dsl.fields import DjangoDSLField

from .behaviors import Labelled, WithBaseModel


class Datasource(Labelled, WithBaseModel):
    """Datasource gets data from the database.

    It uses self.base_model.objects.all(), unless there's something in
    dsl_query field.

    If there is, it compiles that DSL and uses
    self.base_model.objects.filter().
    """

    dsl_query = DjangoDSLField(
        verbose_name=_("DSL query"))

    distinct = models.BooleanField(
        default=True,
        verbose_name=_("Distinct"),
        help_text=_("Output only distinct records")
    )

    class Meta:
        verbose_name = _("Datasource")
        verbose_name_plural = _("Datasources")
        ordering = ('label',)

    def get_model(self):
        return self.base_model.model_class()

    def get_shortcuts(self):
        return getattr(self.get_model(), 'django_dsl_shortcuts', {})

    def get_filter(self, context=None):
        return compiler.compile(
            self.dsl_query,
            shortcuts=self.get_shortcuts(),
            context=context)
